@@ -1,4 +1,6 @@
 """Implement functionality to collect tasks."""
+from __future__ import annotations
+
 import inspect
 import os
 import sys
@@ -6,17 +8,14 @@ import time
 from importlib import util as importlib_util
 from pathlib import Path
 from typing import Any
-from typing import Dict
 from typing import Generator
 from typing import Iterable
-from typing import List
-from typing import Optional
-from typing import Union
 
 from _pytask.config import hookimpl
 from _pytask.config import IS_FILE_SYSTEM_CASE_SENSITIVE
 from _pytask.console import console
 from _pytask.console import create_summary_panel
+from _pytask.console import format_task_id
 from _pytask.exceptions import CollectionError
 from _pytask.mark_utils import has_marker
 from _pytask.nodes import create_task_name
@@ -74,7 +73,7 @@ def _collect_from_paths(session: Session) -> None:
 
 
 @hookimpl
-def pytask_ignore_collect(path: Path, config: Dict[str, Any]) -> bool:
+def pytask_ignore_collect(path: Path, config: dict[str, Any]) -> bool:
     """Ignore a path during the collection."""
     is_ignored = any(path.match(pattern) for pattern in config["ignore"])
     return is_ignored
@@ -82,8 +81,8 @@ def pytask_ignore_collect(path: Path, config: Dict[str, Any]) -> bool:
 
 @hookimpl
 def pytask_collect_file_protocol(
-    session: Session, path: Path, reports: List[CollectionReport]
-) -> List[CollectionReport]:
+    session: Session, path: Path, reports: list[CollectionReport]
+) -> list[CollectionReport]:
     """Wrap the collection of tasks from a file to collect reports."""
     try:
         reports = session.hook.pytask_collect_file(
@@ -104,14 +103,14 @@ def pytask_collect_file_protocol(
 
 @hookimpl
 def pytask_collect_file(
-    session: Session, path: Path, reports: List[CollectionReport]
-) -> Optional[List[CollectionReport]]:
+    session: Session, path: Path, reports: list[CollectionReport]
+) -> list[CollectionReport] | None:
     """Collect a file."""
     if any(path.match(pattern) for pattern in session.config["task_files"]):
         spec = importlib_util.spec_from_file_location(path.stem, str(path))
 
         if spec is None:
-            raise ImportError(f"Can't find module '{path.stem}' at location {path}.")
+            raise ImportError(f"Can't find module {path.stem!r} at location {path}.")
 
         mod = importlib_util.module_from_spec(spec)
         spec.loader.exec_module(mod)
@@ -140,7 +139,7 @@ def pytask_collect_file(
 @hookimpl
 def pytask_collect_task_protocol(
     session: Session, path: Path, name: str, obj: Any
-) -> Optional[CollectionReport]:
+) -> CollectionReport | None:
     """Start protocol for collecting a task."""
     try:
         session.hook.pytask_collect_task_setup(
@@ -168,7 +167,7 @@ def pytask_collect_task_protocol(
 @hookimpl(trylast=True)
 def pytask_collect_task(
     session: Session, path: Path, name: str, obj: Any
-) -> Optional[PythonFunctionTask]:
+) -> PythonFunctionTask | None:
     """Collect a task which is a function.
 
     There is some discussion on how to detect functions in this `thread
@@ -185,17 +184,22 @@ def pytask_collect_task(
 
 
 _TEMPLATE_ERROR: str = (
-    "The provided path of the dependency/product in the marker is {}, but the path of "
-    "the file on disk is {}. Case-sensitive file systems would raise an error.\n\n"
+    "The provided path of the dependency/product in the marker is\n\n{}\n\n, but the "
+    "path of the file on disk is\n\n{}\n\nCase-sensitive file systems would raise an "
+    "error because the upper and lower case format of the paths does not match.\n\n"
     "Please, align the names to ensure reproducibility on case-sensitive file systems "
-    "(often Linux or macOS) or disable this error with 'check_casing_of_paths = false'."
+    "(often Linux or macOS) or disable this error with 'check_casing_of_paths = false' "
+    " in your pytask configuration file.\n\n"
+    "Hint: If parts of the path preceding your project directory are not properly "
+    "formatted, check whether you need to call `.resolve()` on `SRC`, `BLD` or other "
+    "paths created from the `__file__` attribute of a module."
 )
 
 
 @hookimpl(trylast=True)
 def pytask_collect_node(
-    session: Session, path: Path, node: Union[str, Path]
-) -> Optional[FilePathNode]:
+    session: Session, path: Path, node: str | Path
+) -> FilePathNode | None:
     """Collect a node of a task as a :class:`pytask.nodes.FilePathNode`.
 
     Strings are assumed to be paths. This might be a strict assumption, but since this
@@ -251,7 +255,7 @@ def _not_ignored_paths(
 
     Parameters
     ----------
-    paths : List[pathlib.Path]
+    paths : Iterable[pathlib.Path]
         List of paths from which tasks are collected.
     session : _pytask.session.Session
         The session.
@@ -272,7 +276,7 @@ def _not_ignored_paths(
 
 
 @hookimpl(trylast=True)
-def pytask_collect_modify_tasks(tasks: List[MetaTask]) -> None:
+def pytask_collect_modify_tasks(tasks: list[MetaTask]) -> None:
     """Given all tasks, assign a short uniquely identifiable name to each task.
 
     The shorter ids are necessary to display
@@ -285,8 +289,8 @@ def pytask_collect_modify_tasks(tasks: List[MetaTask]) -> None:
 
 
 def _find_shortest_uniquely_identifiable_name_for_tasks(
-    tasks: List[MetaTask],
-) -> Dict[str, str]:
+    tasks: list[MetaTask],
+) -> dict[str, str]:
     """Find the shortest uniquely identifiable name for tasks.
 
     The shortest unique id consists of the module name plus the base name (e.g. function
@@ -319,7 +323,7 @@ def _find_shortest_uniquely_identifiable_name_for_tasks(
 
 @hookimpl
 def pytask_collect_log(
-    session: Session, reports: List[CollectionReport], tasks: List[PythonFunctionTask]
+    session: Session, reports: list[CollectionReport], tasks: list[PythonFunctionTask]
 ) -> None:
     """Log collection."""
     session.collection_end = time.time()
@@ -340,7 +344,12 @@ def pytask_collect_log(
             if report.node is None:
                 header = "Error"
             else:
-                short_name = reduce_node_name(report.node, session.config["paths"])
+                if isinstance(report.node, MetaTask):
+                    short_name = format_task_id(
+                        report.node, editor_url_scheme="no_link", short_name=True
+                    )
+                else:
+                    short_name = reduce_node_name(report.node, session.config["paths"])
                 header = f"Could not collect {short_name}"
 
             console.rule(
